from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.citation_data import CitationData





T = TypeVar("T", bound="ModelCitationToolToolResponses")



@_attrs_define
class ModelCitationToolToolResponses:
    """ 
     """

    additional_properties: dict[str, CitationData] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.citation_data import CitationData
        
        field_dict: dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()


        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.citation_data import CitationData
        d = dict(src_dict)
        model_citation_tool_tool_responses = cls(
        )


        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = CitationData.from_dict(prop_dict)



            additional_properties[prop_name] = additional_property

        model_citation_tool_tool_responses.additional_properties = additional_properties
        return model_citation_tool_tool_responses

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> CitationData:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: CitationData) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.copy_document_result import CopyDocumentResult





T = TypeVar("T", bound="CopyResponse")



@_attrs_define
class CopyResponse:
    """ 
        Attributes:
            detail (str):
            documents_copied (int | Unset):  Default: 0.
            results (list[CopyDocumentResult] | Unset):
     """

    detail: str
    documents_copied: int | Unset = 0
    results: list[CopyDocumentResult] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.copy_document_result import CopyDocumentResult
        detail = self.detail

        documents_copied = self.documents_copied

        results: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.results, Unset):
            results = []
            for results_item_data in self.results:
                results_item = results_item_data.to_dict()
                results.append(results_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "detail": detail,
        })
        if documents_copied is not UNSET:
            field_dict["documents_copied"] = documents_copied
        if results is not UNSET:
            field_dict["results"] = results

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.copy_document_result import CopyDocumentResult
        d = dict(src_dict)
        detail = d.pop("detail")

        documents_copied = d.pop("documents_copied", UNSET)

        _results = d.pop("results", UNSET)
        results: list[CopyDocumentResult] | Unset = UNSET
        if _results is not UNSET:
            results = []
            for results_item_data in _results:
                results_item = CopyDocumentResult.from_dict(results_item_data)



                results.append(results_item)


        copy_response = cls(
            detail=detail,
            documents_copied=documents_copied,
            results=results,
        )


        copy_response.additional_properties = d
        return copy_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum

class TagFormatType(str, Enum):
    CHECKBOX = "checkbox"
    DATE = "date"
    NUMBER = "number"
    SEARCH = "search"
    SELECT = "select"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="SearchInsideDocumentsDetail")



@_attrs_define
class SearchInsideDocumentsDetail:
    """ Detail for a search_inside_documents tool call.

        Attributes:
            query (str): Search query (truncated to 80 chars)
            search_mode (str): Search mode: hybrid, semantic, keyword
            tool (Literal['search_inside_documents'] | Unset):  Default: 'search_inside_documents'.
            label (str | Unset): Display label for the tool step Default: 'Searching documents'.
            icon (str | Unset): Lucide icon name for the tool step Default: 'search'.
            summary (None | str | Unset): Human-readable summary (e.g. 'Search: "query" (hybrid)')
     """

    query: str
    search_mode: str
    tool: Literal['search_inside_documents'] | Unset = 'search_inside_documents'
    label: str | Unset = 'Searching documents'
    icon: str | Unset = 'search'
    summary: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        query = self.query

        search_mode = self.search_mode

        tool = self.tool

        label = self.label

        icon = self.icon

        summary: None | str | Unset
        if isinstance(self.summary, Unset):
            summary = UNSET
        else:
            summary = self.summary


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "query": query,
            "search_mode": search_mode,
        })
        if tool is not UNSET:
            field_dict["tool"] = tool
        if label is not UNSET:
            field_dict["label"] = label
        if icon is not UNSET:
            field_dict["icon"] = icon
        if summary is not UNSET:
            field_dict["summary"] = summary

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        query = d.pop("query")

        search_mode = d.pop("search_mode")

        tool = cast(Literal['search_inside_documents'] | Unset , d.pop("tool", UNSET))
        if tool != 'search_inside_documents'and not isinstance(tool, Unset):
            raise ValueError(f"tool must match const 'search_inside_documents', got '{tool}'")

        label = d.pop("label", UNSET)

        icon = d.pop("icon", UNSET)

        def _parse_summary(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        summary = _parse_summary(d.pop("summary", UNSET))


        search_inside_documents_detail = cls(
            query=query,
            search_mode=search_mode,
            tool=tool,
            label=label,
            icon=icon,
            summary=summary,
        )


        search_inside_documents_detail.additional_properties = d
        return search_inside_documents_detail

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast






T = TypeVar("T", bound="GenerateAnnotationsRequest")



@_attrs_define
class GenerateAnnotationsRequest:
    """ Request to generate AI annotations for documents using tag instructions.

    Creates a matrix of (docs × tags) annotations. Each tag's name is used as
    the instruction/question to answer about each document.

        Attributes:
            tag_ext_ids (list[str]):
            doc_ext_ids (list[str]):
     """

    tag_ext_ids: list[str]
    doc_ext_ids: list[str]





    def to_dict(self) -> dict[str, Any]:
        tag_ext_ids = self.tag_ext_ids



        doc_ext_ids = self.doc_ext_ids




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "tag_ext_ids": tag_ext_ids,
            "doc_ext_ids": doc_ext_ids,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        tag_ext_ids = cast(list[str], d.pop("tag_ext_ids"))


        doc_ext_ids = cast(list[str], d.pop("doc_ext_ids"))


        generate_annotations_request = cls(
            tag_ext_ids=tag_ext_ids,
            doc_ext_ids=doc_ext_ids,
        )

        return generate_annotations_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.task_update_message_status import TaskUpdateMessageStatus
from ..types import UNSET, Unset
from typing import Literal, cast






T = TypeVar("T", bound="TaskUpdateMessage")



@_attrs_define
class TaskUpdateMessage:
    """ Document processing progress update.

        Attributes:
            doc_ext_id (str):
            workspace_ext_id (str):
            file_name (str):
            status (TaskUpdateMessageStatus):
            progress (int):
            type_ (Literal['task_update'] | Unset):  Default: 'task_update'.
     """

    doc_ext_id: str
    workspace_ext_id: str
    file_name: str
    status: TaskUpdateMessageStatus
    progress: int
    type_: Literal['task_update'] | Unset = 'task_update'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_ext_id = self.doc_ext_id

        workspace_ext_id = self.workspace_ext_id

        file_name = self.file_name

        status = self.status.value

        progress = self.progress

        type_ = self.type_


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "doc_ext_id": doc_ext_id,
            "workspace_ext_id": workspace_ext_id,
            "file_name": file_name,
            "status": status,
            "progress": progress,
        })
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        doc_ext_id = d.pop("doc_ext_id")

        workspace_ext_id = d.pop("workspace_ext_id")

        file_name = d.pop("file_name")

        status = TaskUpdateMessageStatus(d.pop("status"))




        progress = d.pop("progress")

        type_ = cast(Literal['task_update'] | Unset , d.pop("type", UNSET))
        if type_ != 'task_update'and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'task_update', got '{type_}'")

        task_update_message = cls(
            doc_ext_id=doc_ext_id,
            workspace_ext_id=workspace_ext_id,
            file_name=file_name,
            status=status,
            progress=progress,
            type_=type_,
        )


        task_update_message.additional_properties = d
        return task_update_message

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.review_llm_config_api_type import ReviewLLMConfigApiType
from ..types import UNSET, Unset






T = TypeVar("T", bound="ReviewLLMConfig")



@_attrs_define
class ReviewLLMConfig:
    r""" Configuration for ReviewLLM - reviews agent draft answers against source material.

        Attributes:
            api_type (ReviewLLMConfigApiType | Unset): The inference type (local or remote). Default:
                ReviewLLMConfigApiType.REMOTE.
            enable_thinking (bool | Unset): Enable reasoning/thinking mode. Default: False.
            model_name (str | Unset): The model for reviewing agent draft answers. Defaults to reasoning model. Default:
                'dummy'.
            system_instruction (str | Unset): System instruction for reviewing agent draft answers against source material.
                Default: 'You are reviewing a draft answer prepared by a research agent. The draft was written based on
                summaries, but you now have access to the full source material.\n\nYour task:\n1. Review the draft answer
                against the source material provided\n2. Verify claims are supported by the sources\n3. Correct any inaccuracies
                or unsupported statements\n4. Add any important details from the sources that were missed\n5. Maintain formal,
                objective tone appropriate for professional contexts\n\nIf information is not found in the sources but the draft
                answer is based on web search results or general knowledge, preserve it and ensure it is clearly labelled as
                such. Only flag missing source support for claims that purport to come from the documents.\nDo not add inline
                citation markers - the system handles citations automatically.'.
            temperature (float | Unset): Temperature for review. Default: 0.1.
            max_tokens (int | Unset): Maximum tokens for review response. Default: 5000.
            max_char_size_to_answer (int | Unset): Maximum character size for review context. Default: 200000.
     """

    api_type: ReviewLLMConfigApiType | Unset = ReviewLLMConfigApiType.REMOTE
    enable_thinking: bool | Unset = False
    model_name: str | Unset = 'dummy'
    system_instruction: str | Unset = 'You are reviewing a draft answer prepared by a research agent. The draft was written based on summaries, but you now have access to the full source material.\n\nYour task:\n1. Review the draft answer against the source material provided\n2. Verify claims are supported by the sources\n3. Correct any inaccuracies or unsupported statements\n4. Add any important details from the sources that were missed\n5. Maintain formal, objective tone appropriate for professional contexts\n\nIf information is not found in the sources but the draft answer is based on web search results or general knowledge, preserve it and ensure it is clearly labelled as such. Only flag missing source support for claims that purport to come from the documents.\nDo not add inline citation markers - the system handles citations automatically.'
    temperature: float | Unset = 0.1
    max_tokens: int | Unset = 5000
    max_char_size_to_answer: int | Unset = 200000
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        api_type: str | Unset = UNSET
        if not isinstance(self.api_type, Unset):
            api_type = self.api_type.value


        enable_thinking = self.enable_thinking

        model_name = self.model_name

        system_instruction = self.system_instruction

        temperature = self.temperature

        max_tokens = self.max_tokens

        max_char_size_to_answer = self.max_char_size_to_answer


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if api_type is not UNSET:
            field_dict["API_TYPE"] = api_type
        if enable_thinking is not UNSET:
            field_dict["ENABLE_THINKING"] = enable_thinking
        if model_name is not UNSET:
            field_dict["MODEL_NAME"] = model_name
        if system_instruction is not UNSET:
            field_dict["SYSTEM_INSTRUCTION"] = system_instruction
        if temperature is not UNSET:
            field_dict["TEMPERATURE"] = temperature
        if max_tokens is not UNSET:
            field_dict["MAX_TOKENS"] = max_tokens
        if max_char_size_to_answer is not UNSET:
            field_dict["MAX_CHAR_SIZE_TO_ANSWER"] = max_char_size_to_answer

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _api_type = d.pop("API_TYPE", UNSET)
        api_type: ReviewLLMConfigApiType | Unset
        if isinstance(_api_type,  Unset):
            api_type = UNSET
        else:
            api_type = ReviewLLMConfigApiType(_api_type)




        enable_thinking = d.pop("ENABLE_THINKING", UNSET)

        model_name = d.pop("MODEL_NAME", UNSET)

        system_instruction = d.pop("SYSTEM_INSTRUCTION", UNSET)

        temperature = d.pop("TEMPERATURE", UNSET)

        max_tokens = d.pop("MAX_TOKENS", UNSET)

        max_char_size_to_answer = d.pop("MAX_CHAR_SIZE_TO_ANSWER", UNSET)

        review_llm_config = cls(
            api_type=api_type,
            enable_thinking=enable_thinking,
            model_name=model_name,
            system_instruction=system_instruction,
            temperature=temperature,
            max_tokens=max_tokens,
            max_char_size_to_answer=max_char_size_to_answer,
        )


        review_llm_config.additional_properties = d
        return review_llm_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.embedder_config_api_type import EmbedderConfigApiType
from ..types import UNSET, Unset






T = TypeVar("T", bound="EmbedderConfig")



@_attrs_define
class EmbedderConfig:
    r""" 
        Attributes:
            model_name (str | Unset): The name of the embedder model. Default: 'Qwen/Qwen3-Embedding-0.6B'.
            api_type (EmbedderConfigApiType | Unset): The inference type (local or remote). Default:
                EmbedderConfigApiType.LOCAL.
            batch_size (int | Unset): Smaller batch size for better parallelization. Default: 128.
            max_concurrent_requests (int | Unset): Adjust concurrency level as needed. Default: 64.
            document_prefix (str | Unset): Prefix for document embeddings. Empty for Qwen3-style models. Default: ''.
            query_prefix (str | Unset): Instruction prefix for query embeddings. Used with Qwen3-style instruction-aware
                models. Default: 'Instruct: Represent the user search query for retrieving relevant documents.\nQuery:'.
     """

    model_name: str | Unset = 'Qwen/Qwen3-Embedding-0.6B'
    api_type: EmbedderConfigApiType | Unset = EmbedderConfigApiType.LOCAL
    batch_size: int | Unset = 128
    max_concurrent_requests: int | Unset = 64
    document_prefix: str | Unset = ''
    query_prefix: str | Unset = 'Instruct: Represent the user search query for retrieving relevant documents.\nQuery:'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        model_name = self.model_name

        api_type: str | Unset = UNSET
        if not isinstance(self.api_type, Unset):
            api_type = self.api_type.value


        batch_size = self.batch_size

        max_concurrent_requests = self.max_concurrent_requests

        document_prefix = self.document_prefix

        query_prefix = self.query_prefix


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if model_name is not UNSET:
            field_dict["MODEL_NAME"] = model_name
        if api_type is not UNSET:
            field_dict["API_TYPE"] = api_type
        if batch_size is not UNSET:
            field_dict["BATCH_SIZE"] = batch_size
        if max_concurrent_requests is not UNSET:
            field_dict["MAX_CONCURRENT_REQUESTS"] = max_concurrent_requests
        if document_prefix is not UNSET:
            field_dict["DOCUMENT_PREFIX"] = document_prefix
        if query_prefix is not UNSET:
            field_dict["QUERY_PREFIX"] = query_prefix

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        model_name = d.pop("MODEL_NAME", UNSET)

        _api_type = d.pop("API_TYPE", UNSET)
        api_type: EmbedderConfigApiType | Unset
        if isinstance(_api_type,  Unset):
            api_type = UNSET
        else:
            api_type = EmbedderConfigApiType(_api_type)




        batch_size = d.pop("BATCH_SIZE", UNSET)

        max_concurrent_requests = d.pop("MAX_CONCURRENT_REQUESTS", UNSET)

        document_prefix = d.pop("DOCUMENT_PREFIX", UNSET)

        query_prefix = d.pop("QUERY_PREFIX", UNSET)

        embedder_config = cls(
            model_name=model_name,
            api_type=api_type,
            batch_size=batch_size,
            max_concurrent_requests=max_concurrent_requests,
            document_prefix=document_prefix,
            query_prefix=query_prefix,
        )


        embedder_config.additional_properties = d
        return embedder_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="SubscriptionStatusResponse")



@_attrs_define
class SubscriptionStatusResponse:
    """ Subscription status response - unified model for all subscription states.

    Status is one of the tier names configured in STRIPE_TIERS, plus system tiers:
    - Paid tiers (e.g., "pro", "pro_plus"): Active Stripe subscription
    - "trial": User is on trial (has trial_expires, days_remaining)
    - "dev": Developer access
    - "restricted": No access (trial expired or no subscription)
    - "inactive": Expired/canceled/unpaid Stripe subscription

        Attributes:
            status (str):
            trial_expires (int | None | Unset):
            days_remaining (int | None | Unset):
            price_id (None | str | Unset):
            plan (None | str | Unset):
            amount (int | None | Unset):
            currency (None | str | Unset):
            current_period_end (int | None | Unset):
            cancel_at_period_end (bool | None | Unset):
            portal_url (None | str | Unset):
            spend (float | None | Unset):
            max_budget (float | None | Unset):
            budget_reset_at (int | None | Unset):
            total_files_mb (float | None | Unset):
            storage_quota_mb (int | None | Unset):
     """

    status: str
    trial_expires: int | None | Unset = UNSET
    days_remaining: int | None | Unset = UNSET
    price_id: None | str | Unset = UNSET
    plan: None | str | Unset = UNSET
    amount: int | None | Unset = UNSET
    currency: None | str | Unset = UNSET
    current_period_end: int | None | Unset = UNSET
    cancel_at_period_end: bool | None | Unset = UNSET
    portal_url: None | str | Unset = UNSET
    spend: float | None | Unset = UNSET
    max_budget: float | None | Unset = UNSET
    budget_reset_at: int | None | Unset = UNSET
    total_files_mb: float | None | Unset = UNSET
    storage_quota_mb: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        status = self.status

        trial_expires: int | None | Unset
        if isinstance(self.trial_expires, Unset):
            trial_expires = UNSET
        else:
            trial_expires = self.trial_expires

        days_remaining: int | None | Unset
        if isinstance(self.days_remaining, Unset):
            days_remaining = UNSET
        else:
            days_remaining = self.days_remaining

        price_id: None | str | Unset
        if isinstance(self.price_id, Unset):
            price_id = UNSET
        else:
            price_id = self.price_id

        plan: None | str | Unset
        if isinstance(self.plan, Unset):
            plan = UNSET
        else:
            plan = self.plan

        amount: int | None | Unset
        if isinstance(self.amount, Unset):
            amount = UNSET
        else:
            amount = self.amount

        currency: None | str | Unset
        if isinstance(self.currency, Unset):
            currency = UNSET
        else:
            currency = self.currency

        current_period_end: int | None | Unset
        if isinstance(self.current_period_end, Unset):
            current_period_end = UNSET
        else:
            current_period_end = self.current_period_end

        cancel_at_period_end: bool | None | Unset
        if isinstance(self.cancel_at_period_end, Unset):
            cancel_at_period_end = UNSET
        else:
            cancel_at_period_end = self.cancel_at_period_end

        portal_url: None | str | Unset
        if isinstance(self.portal_url, Unset):
            portal_url = UNSET
        else:
            portal_url = self.portal_url

        spend: float | None | Unset
        if isinstance(self.spend, Unset):
            spend = UNSET
        else:
            spend = self.spend

        max_budget: float | None | Unset
        if isinstance(self.max_budget, Unset):
            max_budget = UNSET
        else:
            max_budget = self.max_budget

        budget_reset_at: int | None | Unset
        if isinstance(self.budget_reset_at, Unset):
            budget_reset_at = UNSET
        else:
            budget_reset_at = self.budget_reset_at

        total_files_mb: float | None | Unset
        if isinstance(self.total_files_mb, Unset):
            total_files_mb = UNSET
        else:
            total_files_mb = self.total_files_mb

        storage_quota_mb: int | None | Unset
        if isinstance(self.storage_quota_mb, Unset):
            storage_quota_mb = UNSET
        else:
            storage_quota_mb = self.storage_quota_mb


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "status": status,
        })
        if trial_expires is not UNSET:
            field_dict["trial_expires"] = trial_expires
        if days_remaining is not UNSET:
            field_dict["days_remaining"] = days_remaining
        if price_id is not UNSET:
            field_dict["price_id"] = price_id
        if plan is not UNSET:
            field_dict["plan"] = plan
        if amount is not UNSET:
            field_dict["amount"] = amount
        if currency is not UNSET:
            field_dict["currency"] = currency
        if current_period_end is not UNSET:
            field_dict["current_period_end"] = current_period_end
        if cancel_at_period_end is not UNSET:
            field_dict["cancel_at_period_end"] = cancel_at_period_end
        if portal_url is not UNSET:
            field_dict["portal_url"] = portal_url
        if spend is not UNSET:
            field_dict["spend"] = spend
        if max_budget is not UNSET:
            field_dict["max_budget"] = max_budget
        if budget_reset_at is not UNSET:
            field_dict["budget_reset_at"] = budget_reset_at
        if total_files_mb is not UNSET:
            field_dict["total_files_mb"] = total_files_mb
        if storage_quota_mb is not UNSET:
            field_dict["storage_quota_mb"] = storage_quota_mb

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = d.pop("status")

        def _parse_trial_expires(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        trial_expires = _parse_trial_expires(d.pop("trial_expires", UNSET))


        def _parse_days_remaining(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        days_remaining = _parse_days_remaining(d.pop("days_remaining", UNSET))


        def _parse_price_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        price_id = _parse_price_id(d.pop("price_id", UNSET))


        def _parse_plan(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        plan = _parse_plan(d.pop("plan", UNSET))


        def _parse_amount(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        amount = _parse_amount(d.pop("amount", UNSET))


        def _parse_currency(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        currency = _parse_currency(d.pop("currency", UNSET))


        def _parse_current_period_end(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        current_period_end = _parse_current_period_end(d.pop("current_period_end", UNSET))


        def _parse_cancel_at_period_end(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        cancel_at_period_end = _parse_cancel_at_period_end(d.pop("cancel_at_period_end", UNSET))


        def _parse_portal_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        portal_url = _parse_portal_url(d.pop("portal_url", UNSET))


        def _parse_spend(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        spend = _parse_spend(d.pop("spend", UNSET))


        def _parse_max_budget(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        max_budget = _parse_max_budget(d.pop("max_budget", UNSET))


        def _parse_budget_reset_at(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        budget_reset_at = _parse_budget_reset_at(d.pop("budget_reset_at", UNSET))


        def _parse_total_files_mb(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        total_files_mb = _parse_total_files_mb(d.pop("total_files_mb", UNSET))


        def _parse_storage_quota_mb(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        storage_quota_mb = _parse_storage_quota_mb(d.pop("storage_quota_mb", UNSET))


        subscription_status_response = cls(
            status=status,
            trial_expires=trial_expires,
            days_remaining=days_remaining,
            price_id=price_id,
            plan=plan,
            amount=amount,
            currency=currency,
            current_period_end=current_period_end,
            cancel_at_period_end=cancel_at_period_end,
            portal_url=portal_url,
            spend=spend,
            max_budget=max_budget,
            budget_reset_at=budget_reset_at,
            total_files_mb=total_files_mb,
            storage_quota_mb=storage_quota_mb,
        )


        subscription_status_response.additional_properties = d
        return subscription_status_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Authentication helpers combining crypto with autogenerated API functions."""

from __future__ import annotations

import base64
import time

from arbi_client.api.user import login_user, register_user
from arbi_client.client import Client
from arbi_client.models.login_request import LoginRequest
from arbi_client.models.register_request import RegisterRequest
from arbi_client.sdk._types import SessionState, UserIdentity, WorkspaceInfo
from arbi_client.sdk.crypto import create_login_signature, generate_keypair

# ---------------------------------------------------------------------------
# Agent identity helpers
# ---------------------------------------------------------------------------


def _build_agent_identity(
    agent_email: str,
    signing_key: bytes,
) -> UserIdentity:
    """Build a UserIdentity for a persistent agent from its Ed25519 signing key.

    Derives X25519 encryption keys from the Ed25519 key (same curve conversion
    used during registration).
    """
    from nacl.signing import SigningKey

    sk = SigningKey(signing_key[:32])  # seed is first 32 bytes
    signing_pub_b64 = base64.b64encode(bytes(sk.verify_key)).decode()
    x25519_priv = sk.to_curve25519_private_key()
    enc_pub_b64 = base64.b64encode(bytes(x25519_priv.public_key)).decode()
    enc_priv_bytes = bytes(x25519_priv)

    return UserIdentity(
        email=agent_email,
        external_id="",  # filled in from login response
        given_name="",
        family_name="",
        signing_private_key=bytes(sk),
        signing_public_key_b64=signing_pub_b64,
        encryption_private_key=enc_priv_bytes,
        encryption_public_key_b64=enc_pub_b64,
    )


# ---------------------------------------------------------------------------
# Register
# ---------------------------------------------------------------------------


def register(
    client: Client,
    *,
    email: str,
    password: str,
    given_name: str,
    family_name: str,
    deployment_domain: str,
    verification_credential: str,
) -> UserIdentity:
    """Register a new user and return their identity (keys derived from password).

    Does **not** log in — call :func:`login` or :func:`login_with_identity` afterwards.
    """
    signing_pub_b64, signing_priv, enc_pub_b64, enc_priv = generate_keypair(password, email, deployment_domain)

    register_user.sync(
        client=client,
        body=RegisterRequest(
            email=email,
            signing_key=signing_pub_b64,
            verification_credential=verification_credential,
            given_name=given_name,
            family_name=family_name,
        ),
    )

    # Login to get the user's external_id
    sig_b64, timestamp = create_login_signature(email, signing_priv)
    login_resp = login_user.sync(
        client=client,
        body=LoginRequest(email=email, signature=sig_b64, timestamp=timestamp),
    )
    if login_resp is None or not hasattr(login_resp, "access_token"):
        raise RuntimeError(f"Login after registration failed: {login_resp}")

    return UserIdentity(
        email=email,
        external_id=login_resp.user.external_id,
        given_name=given_name,
        family_name=family_name,
        signing_private_key=signing_priv,
        signing_public_key_b64=signing_pub_b64,
        encryption_private_key=enc_priv,
        encryption_public_key_b64=enc_pub_b64,
    )


async def aregister(
    client: Client,
    *,
    email: str,
    password: str,
    given_name: str,
    family_name: str,
    deployment_domain: str,
    verification_credential: str,
) -> UserIdentity:
    """Async variant of :func:`register`."""
    signing_pub_b64, signing_priv, enc_pub_b64, enc_priv = generate_keypair(password, email, deployment_domain)

    await register_user.asyncio(
        client=client,
        body=RegisterRequest(
            email=email,
            signing_key=signing_pub_b64,
            verification_credential=verification_credential,
            given_name=given_name,
            family_name=family_name,
        ),
    )

    sig_b64, timestamp = create_login_signature(email, signing_priv)
    login_resp = await login_user.asyncio(
        client=client,
        body=LoginRequest(email=email, signature=sig_b64, timestamp=timestamp),
    )
    if login_resp is None or not hasattr(login_resp, "access_token"):
        raise RuntimeError(f"Login after registration failed: {login_resp}")

    return UserIdentity(
        email=email,
        external_id=login_resp.user.external_id,
        given_name=given_name,
        family_name=family_name,
        signing_private_key=signing_priv,
        signing_public_key_b64=signing_pub_b64,
        encryption_private_key=enc_priv,
        encryption_public_key_b64=enc_pub_b64,
    )


# ---------------------------------------------------------------------------
# Login
# ---------------------------------------------------------------------------


def _extract_workspaces(login_resp) -> list[WorkspaceInfo]:
    """Extract workspace info from login response for header caching."""
    from arbi_client.types import UNSET

    raw = getattr(login_resp, "workspaces", UNSET)
    if raw is UNSET or raw is None:
        return []
    return [
        WorkspaceInfo(
            external_id=ws.additional_properties.get("external_id", ""),
            wrapped_key=ws.additional_properties.get("wrapped_key"),
        )
        for ws in raw
        if hasattr(ws, "additional_properties")
    ]


def login(
    client: Client,
    *,
    email: str,
    password: str,
    deployment_domain: str,
) -> tuple[UserIdentity, SessionState, list[WorkspaceInfo]]:
    """Derive keys from password, log in, and return identity + session state + workspaces."""
    signing_pub_b64, signing_priv, enc_pub_b64, enc_priv = generate_keypair(password, email, deployment_domain)

    sig_b64, timestamp = create_login_signature(email, signing_priv)

    login_resp = login_user.sync(
        client=client,
        body=LoginRequest(email=email, signature=sig_b64, timestamp=timestamp),
    )
    if login_resp is None or not hasattr(login_resp, "access_token"):
        raise RuntimeError(f"Login failed: {login_resp}")

    identity = UserIdentity(
        email=email,
        external_id=login_resp.user.external_id,
        given_name=login_resp.user.given_name or "",
        family_name=login_resp.user.family_name or "",
        signing_private_key=signing_priv,
        signing_public_key_b64=signing_pub_b64,
        encryption_private_key=enc_priv,
        encryption_public_key_b64=enc_pub_b64,
    )

    session = SessionState(
        access_token=login_resp.access_token,
        session_public_key_b64=login_resp.session_key,
    )

    return identity, session, _extract_workspaces(login_resp)


async def alogin(
    client: Client,
    *,
    email: str,
    password: str,
    deployment_domain: str,
) -> tuple[UserIdentity, SessionState, list[WorkspaceInfo]]:
    """Async variant of :func:`login`."""
    signing_pub_b64, signing_priv, enc_pub_b64, enc_priv = generate_keypair(password, email, deployment_domain)

    sig_b64, timestamp = create_login_signature(email, signing_priv)

    login_resp = await login_user.asyncio(
        client=client,
        body=LoginRequest(email=email, signature=sig_b64, timestamp=timestamp),
    )
    if login_resp is None or not hasattr(login_resp, "access_token"):
        raise RuntimeError(f"Login failed: {login_resp}")

    identity = UserIdentity(
        email=email,
        external_id=login_resp.user.external_id,
        given_name=login_resp.user.given_name or "",
        family_name=login_resp.user.family_name or "",
        signing_private_key=signing_priv,
        signing_public_key_b64=signing_pub_b64,
        encryption_private_key=enc_priv,
        encryption_public_key_b64=enc_pub_b64,
    )

    session = SessionState(
        access_token=login_resp.access_token,
        session_public_key_b64=login_resp.session_key,
    )

    return identity, session, _extract_workspaces(login_resp)


# ---------------------------------------------------------------------------
# Login with existing identity (re-auth without password)
# ---------------------------------------------------------------------------


def login_with_identity(
    client: Client,
    *,
    identity: UserIdentity,
) -> tuple[SessionState, list[WorkspaceInfo]]:
    """Log in using an existing :class:`UserIdentity` (skips key derivation)."""
    sig_b64, timestamp = create_login_signature(identity.email, identity.signing_private_key)

    login_resp = login_user.sync(
        client=client,
        body=LoginRequest(email=identity.email, signature=sig_b64, timestamp=timestamp),
    )
    if login_resp is None or not hasattr(login_resp, "access_token"):
        raise RuntimeError(f"Login failed: {login_resp}")

    session = SessionState(
        access_token=login_resp.access_token,
        session_public_key_b64=login_resp.session_key,
    )
    return session, _extract_workspaces(login_resp)


async def alogin_with_identity(
    client: Client,
    *,
    identity: UserIdentity,
) -> tuple[SessionState, list[WorkspaceInfo]]:
    """Async variant of :func:`login_with_identity`."""
    sig_b64, timestamp = create_login_signature(identity.email, identity.signing_private_key)

    login_resp = await login_user.asyncio(
        client=client,
        body=LoginRequest(email=identity.email, signature=sig_b64, timestamp=timestamp),
    )
    if login_resp is None or not hasattr(login_resp, "access_token"):
        raise RuntimeError(f"Login failed: {login_resp}")

    session = SessionState(
        access_token=login_resp.access_token,
        session_public_key_b64=login_resp.session_key,
    )
    return session, _extract_workspaces(login_resp)


# ---------------------------------------------------------------------------
# Agent login (persistent agent re-authentication)
# ---------------------------------------------------------------------------


def _prepare_agent_login(agent_email: str, signing_key: bytes) -> tuple[LoginRequest, UserIdentity]:
    """Build the login request body and identity for an agent login."""
    from nacl.signing import SigningKey

    sk = SigningKey(signing_key[:32])
    timestamp = int(time.time())
    message = f"{agent_email}|{timestamp}".encode()
    sig_b64 = base64.b64encode(sk.sign(message).signature).decode()

    body = LoginRequest(email=agent_email, signature=sig_b64, timestamp=timestamp)
    identity = _build_agent_identity(agent_email, signing_key)
    return body, identity


def _finish_agent_login(login_resp, identity: UserIdentity) -> tuple[UserIdentity, SessionState, list[WorkspaceInfo]]:
    """Validate response and build return tuple for agent login."""
    if login_resp is None or not hasattr(login_resp, "access_token"):
        raise RuntimeError(f"Agent login failed: {login_resp}")
    session = SessionState(
        access_token=login_resp.access_token,
        session_public_key_b64=login_resp.session_key,
    )
    return identity, session, _extract_workspaces(login_resp)


def agent_login(
    client: Client,
    *,
    agent_email: str,
    signing_key: bytes,
) -> tuple[UserIdentity, SessionState, list[WorkspaceInfo]]:
    """Log in as a persistent agent using Ed25519 signature.

    The agent's X25519 encryption key is derived from its Ed25519 signing key,
    enabling the same workspace key unwrap flow as regular users.

    Args:
        client: Unauthenticated API client.
        agent_email: Agent's email address (e.g. ``agentname.usr-XXXXXXXX@deploymentdomain``).
        signing_key: 64-byte Ed25519 private key (or 32-byte seed).

    Returns:
        ``(identity, session, workspaces)`` — same shape as :func:`login`.
    """
    body, identity = _prepare_agent_login(agent_email, signing_key)
    login_resp = login_user.sync(client=client, body=body)
    return _finish_agent_login(login_resp, identity)


async def aagent_login(
    client: Client,
    *,
    agent_email: str,
    signing_key: bytes,
) -> tuple[UserIdentity, SessionState, list[WorkspaceInfo]]:
    """Async variant of :func:`agent_login`."""
    body, identity = _prepare_agent_login(agent_email, signing_key)
    login_resp = await login_user.asyncio(client=client, body=body)
    return _finish_agent_login(login_resp, identity)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.doc_sim_pair import DocSimPair





T = TypeVar("T", bound="DocSimResponse")



@_attrs_define
class DocSimResponse:
    """ Response for the near-duplicates endpoint.

        Attributes:
            workspace_ext_id (str):
            pairs (list[DocSimPair]):
     """

    workspace_ext_id: str
    pairs: list[DocSimPair]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.doc_sim_pair import DocSimPair
        workspace_ext_id = self.workspace_ext_id

        pairs = []
        for pairs_item_data in self.pairs:
            pairs_item = pairs_item_data.to_dict()
            pairs.append(pairs_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "workspace_ext_id": workspace_ext_id,
            "pairs": pairs,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.doc_sim_pair import DocSimPair
        d = dict(src_dict)
        workspace_ext_id = d.pop("workspace_ext_id")

        pairs = []
        _pairs = d.pop("pairs")
        for pairs_item_data in (_pairs):
            pairs_item = DocSimPair.from_dict(pairs_item_data)



            pairs.append(pairs_item)


        doc_sim_response = cls(
            workspace_ext_id=workspace_ext_id,
            pairs=pairs,
        )


        doc_sim_response.additional_properties = d
        return doc_sim_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.create_agent_request import CreateAgentRequest
from ...models.http_validation_error import HTTPValidationError
from ...models.user_response import UserResponse
from typing import cast



def _get_kwargs(
    *,
    body: CreateAgentRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/user/agent",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | UserResponse | None:
    if response.status_code == 201:
        response_201 = UserResponse.from_dict(response.json())



        return response_201

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | UserResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: CreateAgentRequest,

) -> Response[HTTPValidationError | UserResponse]:
    """ Create Agent

     Create a persistent agent owned by the current user.

    The frontend generates an Ed25519 keypair from a random seed (the agent's password).
    Only the public key is sent here.  The agent logs in via POST /user/login using its
    synthetic email (``agentname.usr-XXXXXXXX@deploymentdomain``) and a signature derived
    from the seed.

    Workspace access is granted separately via POST /workspace/{id}/users.

    Args:
        body (CreateAgentRequest): Create a persistent agent owned by the current user.

            Used by: POST /user/agent (authenticated, non-agent)

            Frontend generates an Ed25519 keypair from a random seed (the agent's "password").
            Only the public key is sent here; the seed is given to the agent CLI.
            The agent's synthetic email is ``agentname.usr-XXXXXXXX@deploymentdomain``.
            The agent logs in via POST /user/login using this email + Ed25519 signature.

            Agent name must be composed of letters, digits, hyphens, and underscores only
            (must start with a letter or digit) so it is valid as an email local-part.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | UserResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: CreateAgentRequest,

) -> HTTPValidationError | UserResponse | None:
    """ Create Agent

     Create a persistent agent owned by the current user.

    The frontend generates an Ed25519 keypair from a random seed (the agent's password).
    Only the public key is sent here.  The agent logs in via POST /user/login using its
    synthetic email (``agentname.usr-XXXXXXXX@deploymentdomain``) and a signature derived
    from the seed.

    Workspace access is granted separately via POST /workspace/{id}/users.

    Args:
        body (CreateAgentRequest): Create a persistent agent owned by the current user.

            Used by: POST /user/agent (authenticated, non-agent)

            Frontend generates an Ed25519 keypair from a random seed (the agent's "password").
            Only the public key is sent here; the seed is given to the agent CLI.
            The agent's synthetic email is ``agentname.usr-XXXXXXXX@deploymentdomain``.
            The agent logs in via POST /user/login using this email + Ed25519 signature.

            Agent name must be composed of letters, digits, hyphens, and underscores only
            (must start with a letter or digit) so it is valid as an email local-part.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | UserResponse
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: CreateAgentRequest,

) -> Response[HTTPValidationError | UserResponse]:
    """ Create Agent

     Create a persistent agent owned by the current user.

    The frontend generates an Ed25519 keypair from a random seed (the agent's password).
    Only the public key is sent here.  The agent logs in via POST /user/login using its
    synthetic email (``agentname.usr-XXXXXXXX@deploymentdomain``) and a signature derived
    from the seed.

    Workspace access is granted separately via POST /workspace/{id}/users.

    Args:
        body (CreateAgentRequest): Create a persistent agent owned by the current user.

            Used by: POST /user/agent (authenticated, non-agent)

            Frontend generates an Ed25519 keypair from a random seed (the agent's "password").
            Only the public key is sent here; the seed is given to the agent CLI.
            The agent's synthetic email is ``agentname.usr-XXXXXXXX@deploymentdomain``.
            The agent logs in via POST /user/login using this email + Ed25519 signature.

            Agent name must be composed of letters, digits, hyphens, and underscores only
            (must start with a letter or digit) so it is valid as an email local-part.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | UserResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: CreateAgentRequest,

) -> HTTPValidationError | UserResponse | None:
    """ Create Agent

     Create a persistent agent owned by the current user.

    The frontend generates an Ed25519 keypair from a random seed (the agent's password).
    Only the public key is sent here.  The agent logs in via POST /user/login using its
    synthetic email (``agentname.usr-XXXXXXXX@deploymentdomain``) and a signature derived
    from the seed.

    Workspace access is granted separately via POST /workspace/{id}/users.

    Args:
        body (CreateAgentRequest): Create a persistent agent owned by the current user.

            Used by: POST /user/agent (authenticated, non-agent)

            Frontend generates an Ed25519 keypair from a random seed (the agent's "password").
            Only the public key is sent here; the seed is given to the agent CLI.
            The agent's synthetic email is ``agentname.usr-XXXXXXXX@deploymentdomain``.
            The agent logs in via POST /user/login using this email + Ed25519 signature.

            Agent name must be composed of letters, digits, hyphens, and underscores only
            (must start with a letter or digit) so it is valid as an email local-part.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | UserResponse
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.notification_response import NotificationResponse
from ...models.send_message_request import SendMessageRequest
from typing import cast



def _get_kwargs(
    *,
    body: SendMessageRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/notifications/",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | list[NotificationResponse] | None:
    if response.status_code == 201:
        response_201 = []
        _response_201 = response.json()
        for response_201_item_data in (_response_201):
            response_201_item = NotificationResponse.from_dict(response_201_item_data)



            response_201.append(response_201_item)

        return response_201

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | list[NotificationResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: SendMessageRequest,

) -> Response[HTTPValidationError | list[NotificationResponse]]:
    """ Send Messages

     Send E2E encrypted messages to one or more users.

    Each message is encrypted with the recipient's individual shared key.
    Creates bilateral notifications visible to both sender and recipient.
    If recipient is online via WebSocket, delivers in real-time.

    Returns the created notifications.

    Args:
        body (SendMessageRequest): Send E2E encrypted messages to one or more users.

            Each recipient gets the same logical message, encrypted with their
            individual shared key (derived from X25519 DH).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[NotificationResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: SendMessageRequest,

) -> HTTPValidationError | list[NotificationResponse] | None:
    """ Send Messages

     Send E2E encrypted messages to one or more users.

    Each message is encrypted with the recipient's individual shared key.
    Creates bilateral notifications visible to both sender and recipient.
    If recipient is online via WebSocket, delivers in real-time.

    Returns the created notifications.

    Args:
        body (SendMessageRequest): Send E2E encrypted messages to one or more users.

            Each recipient gets the same logical message, encrypted with their
            individual shared key (derived from X25519 DH).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[NotificationResponse]
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: SendMessageRequest,

) -> Response[HTTPValidationError | list[NotificationResponse]]:
    """ Send Messages

     Send E2E encrypted messages to one or more users.

    Each message is encrypted with the recipient's individual shared key.
    Creates bilateral notifications visible to both sender and recipient.
    If recipient is online via WebSocket, delivers in real-time.

    Returns the created notifications.

    Args:
        body (SendMessageRequest): Send E2E encrypted messages to one or more users.

            Each recipient gets the same logical message, encrypted with their
            individual shared key (derived from X25519 DH).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[NotificationResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: SendMessageRequest,

) -> HTTPValidationError | list[NotificationResponse] | None:
    """ Send Messages

     Send E2E encrypted messages to one or more users.

    Each message is encrypted with the recipient's individual shared key.
    Creates bilateral notifications visible to both sender and recipient.
    If recipient is online via WebSocket, delivers in real-time.

    Returns the created notifications.

    Args:
        body (SendMessageRequest): Send E2E encrypted messages to one or more users.

            Each recipient gets the same logical message, encrypted with their
            individual shared key (derived from X25519 DH).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[NotificationResponse]
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Literal, cast






T = TypeVar("T", bound="ViewDocumentPagesDetail")



@_attrs_define
class ViewDocumentPagesDetail:
    """ Detail for a view_document_pages tool call.

        Attributes:
            doc_ext_id (str): Document external ID
            from_page (int): Start page number
            to_page (int): End page number
            tool (Literal['view_document_pages'] | Unset):  Default: 'view_document_pages'.
     """

    doc_ext_id: str
    from_page: int
    to_page: int
    tool: Literal['view_document_pages'] | Unset = 'view_document_pages'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_ext_id = self.doc_ext_id

        from_page = self.from_page

        to_page = self.to_page

        tool = self.tool


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "doc_ext_id": doc_ext_id,
            "from_page": from_page,
            "to_page": to_page,
        })
        if tool is not UNSET:
            field_dict["tool"] = tool

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        doc_ext_id = d.pop("doc_ext_id")

        from_page = d.pop("from_page")

        to_page = d.pop("to_page")

        tool = cast(Literal['view_document_pages'] | Unset , d.pop("tool", UNSET))
        if tool != 'view_document_pages'and not isinstance(tool, Unset):
            raise ValueError(f"tool must match const 'view_document_pages', got '{tool}'")

        view_document_pages_detail = cls(
            doc_ext_id=doc_ext_id,
            from_page=from_page,
            to_page=to_page,
            tool=tool,
        )


        view_document_pages_detail.additional_properties = d
        return view_document_pages_detail

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="ModelCitationConfig")



@_attrs_define
class ModelCitationConfig:
    """ 
        Attributes:
            sim_threashold (float | Unset): Minimum similarity score for a chunk to be cited. Default: 0.5.
            min_char_size_to_answer (int | Unset): Minimum character length to be considered as a statement for citation.
                Default: 30.
            max_numb_citations (int | Unset): Maximum number of citations to return per statement. Default: 5.
            citation_instruction (str | Unset): Instruction for the <Instruct> tag when scoring chunks for citation
                attribution. Default: 'Judge whether the passage contains evidence that directly supports or sources the claim
                made in the query. Only answer yes if the passage provides the basis for the statement'.
     """

    sim_threashold: float | Unset = 0.5
    min_char_size_to_answer: int | Unset = 30
    max_numb_citations: int | Unset = 5
    citation_instruction: str | Unset = 'Judge whether the passage contains evidence that directly supports or sources the claim made in the query. Only answer yes if the passage provides the basis for the statement'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        sim_threashold = self.sim_threashold

        min_char_size_to_answer = self.min_char_size_to_answer

        max_numb_citations = self.max_numb_citations

        citation_instruction = self.citation_instruction


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if sim_threashold is not UNSET:
            field_dict["SIM_THREASHOLD"] = sim_threashold
        if min_char_size_to_answer is not UNSET:
            field_dict["MIN_CHAR_SIZE_TO_ANSWER"] = min_char_size_to_answer
        if max_numb_citations is not UNSET:
            field_dict["MAX_NUMB_CITATIONS"] = max_numb_citations
        if citation_instruction is not UNSET:
            field_dict["CITATION_INSTRUCTION"] = citation_instruction

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        sim_threashold = d.pop("SIM_THREASHOLD", UNSET)

        min_char_size_to_answer = d.pop("MIN_CHAR_SIZE_TO_ANSWER", UNSET)

        max_numb_citations = d.pop("MAX_NUMB_CITATIONS", UNSET)

        citation_instruction = d.pop("CITATION_INSTRUCTION", UNSET)

        model_citation_config = cls(
            sim_threashold=sim_threashold,
            min_char_size_to_answer=min_char_size_to_answer,
            max_numb_citations=max_numb_citations,
            citation_instruction=citation_instruction,
        )


        model_citation_config.additional_properties = d
        return model_citation_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.search_mode import SearchMode
from ..types import UNSET, Unset






T = TypeVar("T", bound="RetrieverConfig")



@_attrs_define
class RetrieverConfig:
    """ 
        Attributes:
            min_retrieval_sim_score (float | Unset): Minimum similarity score for retrieval of a chunk. Default: 0.1.
            max_distinct_documents (int | Unset): Maximum number of distinct documents to search for. Default: 200.
            max_total_chunks_to_retrieve (int | Unset): Maximum total number of chunks to retrieve for all documents
                retrieved. Default: 400.
            group_size (int | Unset): Maximum number of chunks per document for retrieval. Default: 10000.
            search_mode (SearchMode | Unset): Search mode for retrieval.
            hybrid_prefetch_limit (int | Unset): Maximum number of candidates each prefetch (dense and sparse) returns in
                hybrid mode before RRF fusion. Caps the sparse BM25 prefetch to avoid returning tens of thousands of chunks for
                common tokens. Default: 500.
            hybrid_dense_weight (float | Unset): Weight for dense vectors in hybrid mode Default: 0.7.
            hybrid_sparse_weight (float | Unset): Weight for sparse vectors in hybrid mode Default: 0.3.
     """

    min_retrieval_sim_score: float | Unset = 0.1
    max_distinct_documents: int | Unset = 200
    max_total_chunks_to_retrieve: int | Unset = 400
    group_size: int | Unset = 10000
    search_mode: SearchMode | Unset = UNSET
    hybrid_prefetch_limit: int | Unset = 500
    hybrid_dense_weight: float | Unset = 0.7
    hybrid_sparse_weight: float | Unset = 0.3
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        min_retrieval_sim_score = self.min_retrieval_sim_score

        max_distinct_documents = self.max_distinct_documents

        max_total_chunks_to_retrieve = self.max_total_chunks_to_retrieve

        group_size = self.group_size

        search_mode: str | Unset = UNSET
        if not isinstance(self.search_mode, Unset):
            search_mode = self.search_mode.value


        hybrid_prefetch_limit = self.hybrid_prefetch_limit

        hybrid_dense_weight = self.hybrid_dense_weight

        hybrid_sparse_weight = self.hybrid_sparse_weight


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if min_retrieval_sim_score is not UNSET:
            field_dict["MIN_RETRIEVAL_SIM_SCORE"] = min_retrieval_sim_score
        if max_distinct_documents is not UNSET:
            field_dict["MAX_DISTINCT_DOCUMENTS"] = max_distinct_documents
        if max_total_chunks_to_retrieve is not UNSET:
            field_dict["MAX_TOTAL_CHUNKS_TO_RETRIEVE"] = max_total_chunks_to_retrieve
        if group_size is not UNSET:
            field_dict["GROUP_SIZE"] = group_size
        if search_mode is not UNSET:
            field_dict["SEARCH_MODE"] = search_mode
        if hybrid_prefetch_limit is not UNSET:
            field_dict["HYBRID_PREFETCH_LIMIT"] = hybrid_prefetch_limit
        if hybrid_dense_weight is not UNSET:
            field_dict["HYBRID_DENSE_WEIGHT"] = hybrid_dense_weight
        if hybrid_sparse_weight is not UNSET:
            field_dict["HYBRID_SPARSE_WEIGHT"] = hybrid_sparse_weight

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        min_retrieval_sim_score = d.pop("MIN_RETRIEVAL_SIM_SCORE", UNSET)

        max_distinct_documents = d.pop("MAX_DISTINCT_DOCUMENTS", UNSET)

        max_total_chunks_to_retrieve = d.pop("MAX_TOTAL_CHUNKS_TO_RETRIEVE", UNSET)

        group_size = d.pop("GROUP_SIZE", UNSET)

        _search_mode = d.pop("SEARCH_MODE", UNSET)
        search_mode: SearchMode | Unset
        if isinstance(_search_mode,  Unset):
            search_mode = UNSET
        else:
            search_mode = SearchMode(_search_mode)




        hybrid_prefetch_limit = d.pop("HYBRID_PREFETCH_LIMIT", UNSET)

        hybrid_dense_weight = d.pop("HYBRID_DENSE_WEIGHT", UNSET)

        hybrid_sparse_weight = d.pop("HYBRID_SPARSE_WEIGHT", UNSET)

        retriever_config = cls(
            min_retrieval_sim_score=min_retrieval_sim_score,
            max_distinct_documents=max_distinct_documents,
            max_total_chunks_to_retrieve=max_total_chunks_to_retrieve,
            group_size=group_size,
            search_mode=search_mode,
            hybrid_prefetch_limit=hybrid_prefetch_limit,
            hybrid_dense_weight=hybrid_dense_weight,
            hybrid_sparse_weight=hybrid_sparse_weight,
        )


        retriever_config.additional_properties = d
        return retriever_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="OutputTokensDetails")



@_attrs_define
class OutputTokensDetails:
    """ Nested output token details per OpenAI spec.

        Attributes:
            reasoning_tokens (int | Unset): Reasoning tokens used by the model Default: 0.
     """

    reasoning_tokens: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        reasoning_tokens = self.reasoning_tokens


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if reasoning_tokens is not UNSET:
            field_dict["reasoning_tokens"] = reasoning_tokens

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        reasoning_tokens = d.pop("reasoning_tokens", UNSET)

        output_tokens_details = cls(
            reasoning_tokens=reasoning_tokens,
        )


        output_tokens_details.additional_properties = d
        return output_tokens_details

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="WorkspaceUpdateRequest")



@_attrs_define
class WorkspaceUpdateRequest:
    """ 
        Attributes:
            name (None | str | Unset):
            description (None | str | Unset):
            is_public (bool | None | Unset):
     """

    name: None | str | Unset = UNSET
    description: None | str | Unset = UNSET
    is_public: bool | None | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        is_public: bool | None | Unset
        if isinstance(self.is_public, Unset):
            is_public = UNSET
        else:
            is_public = self.is_public


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if is_public is not UNSET:
            field_dict["is_public"] = is_public

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))


        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))


        def _parse_is_public(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_public = _parse_is_public(d.pop("is_public", UNSET))


        workspace_update_request = cls(
            name=name,
            description=description,
            is_public=is_public,
        )

        return workspace_update_request




from enum import Enum

class NotificationType(str, Enum):
    CONTACT_ACCEPTED = "contact_accepted"
    EMAIL_REPLY = "email_reply"
    USER_MESSAGE = "user_message"
    WORKSPACEUSER_ADDED_COLLABORATOR = "workspaceuser_added_collaborator"
    WORKSPACEUSER_ADDED_GUEST = "workspaceuser_added_guest"
    WORKSPACEUSER_ADDED_OWNER = "workspaceuser_added_owner"
    WORKSPACEUSER_REMOVED = "workspaceuser_removed"
    WORKSPACEUSER_UPDATED_COLLABORATOR = "workspaceuser_updated_collaborator"
    WORKSPACEUSER_UPDATED_GUEST = "workspaceuser_updated_guest"
    WORKSPACEUSER_UPDATED_OWNER = "workspaceuser_updated_owner"

    def __str__(self) -> str:
        return str(self.value)



"""Query/chat helper functions for the ARBI SDK.

Provides ``query()`` (simple, returns answer string) and ``query_streaming()``
(returns a full :class:`QueryResult` with typed events and callbacks).
"""

from __future__ import annotations

from collections.abc import Callable
from typing import Any

from arbi_client.models.agent_step_event import AgentStepEvent
from arbi_client.sdk._types import QueryResult, StreamingEvent
from arbi_client.sdk.streaming import run_streaming_query
from arbi_client.sdk.tools import retrieval_chunk as _build_retrieval_chunk


def query(
    base_url: str,
    access_token: str,
    content: str,
    *,
    doc_ext_ids: list[str] | None = None,
    search_mode: str = "hybrid",
    tools: dict | None = None,
    config_ext_id: str | None = None,
    timeout: float = 600.0,
    verify: bool = True,
) -> str:
    """Run a query and return the answer string.

    If *tools* is not provided, a default ``retrieval_chunk`` tool is built
    from *doc_ext_ids* and *search_mode*.
    """
    if tools is None:
        tools = _build_retrieval_chunk(doc_ext_ids=doc_ext_ids, search_mode=search_mode)

    result = run_streaming_query(
        base_url=base_url,
        access_token=access_token,
        query=content,
        tools=tools,
        config_ext_id=config_ext_id,
        timeout=timeout,
        verify=verify,
    )
    return result.content


def query_streaming(
    base_url: str,
    access_token: str,
    content: str,
    *,
    doc_ext_ids: list[str] | None = None,
    search_mode: str = "hybrid",
    tools: dict | None = None,
    config_ext_id: str | None = None,
    on_token: Callable[[str], Any] | None = None,
    on_agent_step: Callable[[AgentStepEvent], Any] | None = None,
    timeout: float = 600.0,
    verify: bool = True,
) -> QueryResult:
    """Run a streaming query with optional per-event callbacks.

    Args:
        base_url: API base URL.
        access_token: Bearer token (workspace-scoped JWT).
        content: The user query.
        doc_ext_ids: Optional list of document IDs to scope search.
        search_mode: Search mode.
        tools: Explicit tools dict.
        config_ext_id: Optional agent config.
        on_token: Called with each content token string as it arrives.
        on_agent_step: Called with a typed :class:`AgentStepEvent` for each step.
        timeout: HTTP timeout in seconds.
        verify: Whether to verify SSL.

    Returns:
        A :class:`QueryResult` with typed events.
    """
    if tools is None:
        tools = _build_retrieval_chunk(doc_ext_ids=doc_ext_ids, search_mode=search_mode)

    def _on_event(event: StreamingEvent) -> None:
        if event.event_type == "response.output_text.delta" and on_token is not None:
            delta = event.data.get("delta", "") if isinstance(event.data, dict) else ""
            if delta:
                on_token(delta)
        elif isinstance(event.data, AgentStepEvent) and on_agent_step is not None:
            on_agent_step(event.data)

    return run_streaming_query(
        base_url=base_url,
        access_token=access_token,
        query=content,
        tools=tools,
        config_ext_id=config_ext_id,
        on_event=_on_event,
        timeout=timeout,
        verify=verify,
    )



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.verify_email_request import VerifyEmailRequest
from ...models.verify_email_response import VerifyEmailResponse
from typing import cast



def _get_kwargs(
    *,
    body: VerifyEmailRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/user/verify-email",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | VerifyEmailResponse | None:
    if response.status_code == 200:
        response_200 = VerifyEmailResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | VerifyEmailResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: VerifyEmailRequest,

) -> Response[HTTPValidationError | VerifyEmailResponse]:
    """ Verify Email

     Send verification email with 3-word code to user.
    Calls central server to send the email.

    Note: Fails silently if email already exists to prevent email enumeration attacks.
    Also returns success even on rate limit/errors to avoid information disclosure.

    Args:
        body (VerifyEmailRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | VerifyEmailResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: VerifyEmailRequest,

) -> HTTPValidationError | VerifyEmailResponse | None:
    """ Verify Email

     Send verification email with 3-word code to user.
    Calls central server to send the email.

    Note: Fails silently if email already exists to prevent email enumeration attacks.
    Also returns success even on rate limit/errors to avoid information disclosure.

    Args:
        body (VerifyEmailRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | VerifyEmailResponse
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: VerifyEmailRequest,

) -> Response[HTTPValidationError | VerifyEmailResponse]:
    """ Verify Email

     Send verification email with 3-word code to user.
    Calls central server to send the email.

    Note: Fails silently if email already exists to prevent email enumeration attacks.
    Also returns success even on rate limit/errors to avoid information disclosure.

    Args:
        body (VerifyEmailRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | VerifyEmailResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: VerifyEmailRequest,

) -> HTTPValidationError | VerifyEmailResponse | None:
    """ Verify Email

     Send verification email with 3-word code to user.
    Calls central server to send the email.

    Note: Fails silently if email already exists to prevent email enumeration attacks.
    Also returns success even on rate limit/errors to avoid information disclosure.

    Args:
        body (VerifyEmailRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | VerifyEmailResponse
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="WebSearchConfig")



@_attrs_define
class WebSearchConfig:
    """ Configuration for web search tools.

        Attributes:
            enabled (bool | Unset): Enable web_search and read_url tools. Default: False.
            save_sources (bool | Unset): Save fetched webpages as workspace documents for citations. Default: True.
     """

    enabled: bool | Unset = False
    save_sources: bool | Unset = True
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        enabled = self.enabled

        save_sources = self.save_sources


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if enabled is not UNSET:
            field_dict["ENABLED"] = enabled
        if save_sources is not UNSET:
            field_dict["SAVE_SOURCES"] = save_sources

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        enabled = d.pop("ENABLED", UNSET)

        save_sources = d.pop("SAVE_SOURCES", UNSET)

        web_search_config = cls(
            enabled=enabled,
            save_sources=save_sources,
        )


        web_search_config.additional_properties = d
        return web_search_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="SessionInfoResponse")



@_attrs_define
class SessionInfoResponse:
    """ Active session info returned by GET /user/sessions.

        Attributes:
            session_id (str):
            ttl (int):
            external_id (None | str | Unset):
            name (None | str | Unset):
            ip_address (None | str | Unset):
            active_workspace (None | str | Unset):
            workspaces (list[str] | Unset):
            status (str | Unset):  Default: 'user'.
     """

    session_id: str
    ttl: int
    external_id: None | str | Unset = UNSET
    name: None | str | Unset = UNSET
    ip_address: None | str | Unset = UNSET
    active_workspace: None | str | Unset = UNSET
    workspaces: list[str] | Unset = UNSET
    status: str | Unset = 'user'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        session_id = self.session_id

        ttl = self.ttl

        external_id: None | str | Unset
        if isinstance(self.external_id, Unset):
            external_id = UNSET
        else:
            external_id = self.external_id

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        ip_address: None | str | Unset
        if isinstance(self.ip_address, Unset):
            ip_address = UNSET
        else:
            ip_address = self.ip_address

        active_workspace: None | str | Unset
        if isinstance(self.active_workspace, Unset):
            active_workspace = UNSET
        else:
            active_workspace = self.active_workspace

        workspaces: list[str] | Unset = UNSET
        if not isinstance(self.workspaces, Unset):
            workspaces = self.workspaces



        status = self.status


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "session_id": session_id,
            "ttl": ttl,
        })
        if external_id is not UNSET:
            field_dict["external_id"] = external_id
        if name is not UNSET:
            field_dict["name"] = name
        if ip_address is not UNSET:
            field_dict["ip_address"] = ip_address
        if active_workspace is not UNSET:
            field_dict["active_workspace"] = active_workspace
        if workspaces is not UNSET:
            field_dict["workspaces"] = workspaces
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        session_id = d.pop("session_id")

        ttl = d.pop("ttl")

        def _parse_external_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        external_id = _parse_external_id(d.pop("external_id", UNSET))


        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))


        def _parse_ip_address(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        ip_address = _parse_ip_address(d.pop("ip_address", UNSET))


        def _parse_active_workspace(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        active_workspace = _parse_active_workspace(d.pop("active_workspace", UNSET))


        workspaces = cast(list[str], d.pop("workspaces", UNSET))


        status = d.pop("status", UNSET)

        session_info_response = cls(
            session_id=session_id,
            ttl=ttl,
            external_id=external_id,
            name=name,
            ip_address=ip_address,
            active_workspace=active_workspace,
            workspaces=workspaces,
            status=status,
        )


        session_info_response.additional_properties = d
        return session_info_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum

class ResponsesAPIResponseStatus(str, Enum):
    CANCELLED = "cancelled"
    COMPLETED = "completed"
    FAILED = "failed"
    INCOMPLETE = "incomplete"
    IN_PROGRESS = "in_progress"
    QUEUED = "queued"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime






T = TypeVar("T", bound="DocMetadata")



@_attrs_define
class DocMetadata:
    """ Structured model for document metadata stored in JSONB column.

        Attributes:
            doc_nature (None | str | Unset):
            doc_author (None | str | Unset):
            doc_subject (None | str | Unset):
            doc_date (datetime.date | None | Unset):
            title (None | str | Unset):
     """

    doc_nature: None | str | Unset = UNSET
    doc_author: None | str | Unset = UNSET
    doc_subject: None | str | Unset = UNSET
    doc_date: datetime.date | None | Unset = UNSET
    title: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_nature: None | str | Unset
        if isinstance(self.doc_nature, Unset):
            doc_nature = UNSET
        else:
            doc_nature = self.doc_nature

        doc_author: None | str | Unset
        if isinstance(self.doc_author, Unset):
            doc_author = UNSET
        else:
            doc_author = self.doc_author

        doc_subject: None | str | Unset
        if isinstance(self.doc_subject, Unset):
            doc_subject = UNSET
        else:
            doc_subject = self.doc_subject

        doc_date: None | str | Unset
        if isinstance(self.doc_date, Unset):
            doc_date = UNSET
        elif isinstance(self.doc_date, datetime.date):
            doc_date = self.doc_date.isoformat()
        else:
            doc_date = self.doc_date

        title: None | str | Unset
        if isinstance(self.title, Unset):
            title = UNSET
        else:
            title = self.title


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if doc_nature is not UNSET:
            field_dict["doc_nature"] = doc_nature
        if doc_author is not UNSET:
            field_dict["doc_author"] = doc_author
        if doc_subject is not UNSET:
            field_dict["doc_subject"] = doc_subject
        if doc_date is not UNSET:
            field_dict["doc_date"] = doc_date
        if title is not UNSET:
            field_dict["title"] = title

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        def _parse_doc_nature(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        doc_nature = _parse_doc_nature(d.pop("doc_nature", UNSET))


        def _parse_doc_author(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        doc_author = _parse_doc_author(d.pop("doc_author", UNSET))


        def _parse_doc_subject(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        doc_subject = _parse_doc_subject(d.pop("doc_subject", UNSET))


        def _parse_doc_date(data: object) -> datetime.date | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                doc_date_type_0 = isoparse(data).date()



                return doc_date_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.date | None | Unset, data)

        doc_date = _parse_doc_date(d.pop("doc_date", UNSET))


        def _parse_title(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        title = _parse_title(d.pop("title", UNSET))


        doc_metadata = cls(
            doc_nature=doc_nature,
            doc_author=doc_author,
            doc_subject=doc_subject,
            doc_date=doc_date,
            title=title,
        )


        doc_metadata.additional_properties = d
        return doc_metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.user_message_event_role import UserMessageEventRole
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
from typing import Literal, cast
import datetime

if TYPE_CHECKING:
  from ..models.user_message_event_tools import UserMessageEventTools





T = TypeVar("T", bound="UserMessageEvent")



@_attrs_define
class UserMessageEvent:
    """ ``arbi.user_message`` — saved user message echoed back at stream start.

        Attributes:
            role (UserMessageEventRole):
            content (str):
            external_id (str):
            created_at (datetime.datetime):
            created_by_ext_id (str):
            conversation_ext_id (str):
            tools (UserMessageEventTools | Unset):
            config_ext_id (None | str | Unset):
            shared (bool | Unset):  Default: False.
            tokens (int | Unset):  Default: 0.
            status (str | Unset):  Default: 'completed'.
            parent_message_ext_id (None | str | Unset):
            type_ (Literal['user_message'] | Unset):  Default: 'user_message'.
     """

    role: UserMessageEventRole
    content: str
    external_id: str
    created_at: datetime.datetime
    created_by_ext_id: str
    conversation_ext_id: str
    tools: UserMessageEventTools | Unset = UNSET
    config_ext_id: None | str | Unset = UNSET
    shared: bool | Unset = False
    tokens: int | Unset = 0
    status: str | Unset = 'completed'
    parent_message_ext_id: None | str | Unset = UNSET
    type_: Literal['user_message'] | Unset = 'user_message'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.user_message_event_tools import UserMessageEventTools
        role = self.role.value

        content = self.content

        external_id = self.external_id

        created_at = self.created_at.isoformat()

        created_by_ext_id = self.created_by_ext_id

        conversation_ext_id = self.conversation_ext_id

        tools: dict[str, Any] | Unset = UNSET
        if not isinstance(self.tools, Unset):
            tools = self.tools.to_dict()

        config_ext_id: None | str | Unset
        if isinstance(self.config_ext_id, Unset):
            config_ext_id = UNSET
        else:
            config_ext_id = self.config_ext_id

        shared = self.shared

        tokens = self.tokens

        status = self.status

        parent_message_ext_id: None | str | Unset
        if isinstance(self.parent_message_ext_id, Unset):
            parent_message_ext_id = UNSET
        else:
            parent_message_ext_id = self.parent_message_ext_id

        type_ = self.type_


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "role": role,
            "content": content,
            "external_id": external_id,
            "created_at": created_at,
            "created_by_ext_id": created_by_ext_id,
            "conversation_ext_id": conversation_ext_id,
        })
        if tools is not UNSET:
            field_dict["tools"] = tools
        if config_ext_id is not UNSET:
            field_dict["config_ext_id"] = config_ext_id
        if shared is not UNSET:
            field_dict["shared"] = shared
        if tokens is not UNSET:
            field_dict["tokens"] = tokens
        if status is not UNSET:
            field_dict["status"] = status
        if parent_message_ext_id is not UNSET:
            field_dict["parent_message_ext_id"] = parent_message_ext_id
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_message_event_tools import UserMessageEventTools
        d = dict(src_dict)
        role = UserMessageEventRole(d.pop("role"))




        content = d.pop("content")

        external_id = d.pop("external_id")

        created_at = isoparse(d.pop("created_at"))




        created_by_ext_id = d.pop("created_by_ext_id")

        conversation_ext_id = d.pop("conversation_ext_id")

        _tools = d.pop("tools", UNSET)
        tools: UserMessageEventTools | Unset
        if isinstance(_tools,  Unset):
            tools = UNSET
        else:
            tools = UserMessageEventTools.from_dict(_tools)




        def _parse_config_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        config_ext_id = _parse_config_ext_id(d.pop("config_ext_id", UNSET))


        shared = d.pop("shared", UNSET)

        tokens = d.pop("tokens", UNSET)

        status = d.pop("status", UNSET)

        def _parse_parent_message_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        parent_message_ext_id = _parse_parent_message_ext_id(d.pop("parent_message_ext_id", UNSET))


        type_ = cast(Literal['user_message'] | Unset , d.pop("type", UNSET))
        if type_ != 'user_message'and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'user_message', got '{type_}'")

        user_message_event = cls(
            role=role,
            content=content,
            external_id=external_id,
            created_at=created_at,
            created_by_ext_id=created_by_ext_id,
            conversation_ext_id=conversation_ext_id,
            tools=tools,
            config_ext_id=config_ext_id,
            shared=shared,
            tokens=tokens,
            status=status,
            parent_message_ext_id=parent_message_ext_id,
            type_=type_,
        )


        user_message_event.additional_properties = d
        return user_message_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.message_recipient import MessageRecipient





T = TypeVar("T", bound="SendMessageRequest")



@_attrs_define
class SendMessageRequest:
    """ Send E2E encrypted messages to one or more users.

    Each recipient gets the same logical message, encrypted with their
    individual shared key (derived from X25519 DH).

        Attributes:
            messages (list[MessageRecipient]):
     """

    messages: list[MessageRecipient]





    def to_dict(self) -> dict[str, Any]:
        from ..models.message_recipient import MessageRecipient
        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "messages": messages,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.message_recipient import MessageRecipient
        d = dict(src_dict)
        messages = []
        _messages = d.pop("messages")
        for messages_item_data in (_messages):
            messages_item = MessageRecipient.from_dict(messages_item_data)



            messages.append(messages_item)


        send_message_request = cls(
            messages=messages,
        )

        return send_message_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.message_input_tools import MessageInputTools





T = TypeVar("T", bound="MessageInput")



@_attrs_define
class MessageInput:
    """ 
        Attributes:
            input_ (str):
            tools (MessageInputTools | Unset):
            workspace_ext_id (str | Unset):  Default: ''.
            previous_response_id (None | str | Unset):
            model (None | str | Unset):
            stream (bool | Unset):  Default: False.
            background (bool | Unset):  Default: False.
            store (bool | Unset):  Default: True.
            instructions (None | str | Unset):
            max_output_tokens (int | None | Unset):
            temperature (float | None | Unset):
            top_p (float | None | Unset):
     """

    input_: str
    tools: MessageInputTools | Unset = UNSET
    workspace_ext_id: str | Unset = ''
    previous_response_id: None | str | Unset = UNSET
    model: None | str | Unset = UNSET
    stream: bool | Unset = False
    background: bool | Unset = False
    store: bool | Unset = True
    instructions: None | str | Unset = UNSET
    max_output_tokens: int | None | Unset = UNSET
    temperature: float | None | Unset = UNSET
    top_p: float | None | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.message_input_tools import MessageInputTools
        input_ = self.input_

        tools: dict[str, Any] | Unset = UNSET
        if not isinstance(self.tools, Unset):
            tools = self.tools.to_dict()

        workspace_ext_id = self.workspace_ext_id

        previous_response_id: None | str | Unset
        if isinstance(self.previous_response_id, Unset):
            previous_response_id = UNSET
        else:
            previous_response_id = self.previous_response_id

        model: None | str | Unset
        if isinstance(self.model, Unset):
            model = UNSET
        else:
            model = self.model

        stream = self.stream

        background = self.background

        store = self.store

        instructions: None | str | Unset
        if isinstance(self.instructions, Unset):
            instructions = UNSET
        else:
            instructions = self.instructions

        max_output_tokens: int | None | Unset
        if isinstance(self.max_output_tokens, Unset):
            max_output_tokens = UNSET
        else:
            max_output_tokens = self.max_output_tokens

        temperature: float | None | Unset
        if isinstance(self.temperature, Unset):
            temperature = UNSET
        else:
            temperature = self.temperature

        top_p: float | None | Unset
        if isinstance(self.top_p, Unset):
            top_p = UNSET
        else:
            top_p = self.top_p


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "input": input_,
        })
        if tools is not UNSET:
            field_dict["tools"] = tools
        if workspace_ext_id is not UNSET:
            field_dict["workspace_ext_id"] = workspace_ext_id
        if previous_response_id is not UNSET:
            field_dict["previous_response_id"] = previous_response_id
        if model is not UNSET:
            field_dict["model"] = model
        if stream is not UNSET:
            field_dict["stream"] = stream
        if background is not UNSET:
            field_dict["background"] = background
        if store is not UNSET:
            field_dict["store"] = store
        if instructions is not UNSET:
            field_dict["instructions"] = instructions
        if max_output_tokens is not UNSET:
            field_dict["max_output_tokens"] = max_output_tokens
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if top_p is not UNSET:
            field_dict["top_p"] = top_p

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.message_input_tools import MessageInputTools
        d = dict(src_dict)
        input_ = d.pop("input")

        _tools = d.pop("tools", UNSET)
        tools: MessageInputTools | Unset
        if isinstance(_tools,  Unset):
            tools = UNSET
        else:
            tools = MessageInputTools.from_dict(_tools)




        workspace_ext_id = d.pop("workspace_ext_id", UNSET)

        def _parse_previous_response_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        previous_response_id = _parse_previous_response_id(d.pop("previous_response_id", UNSET))


        def _parse_model(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        model = _parse_model(d.pop("model", UNSET))


        stream = d.pop("stream", UNSET)

        background = d.pop("background", UNSET)

        store = d.pop("store", UNSET)

        def _parse_instructions(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        instructions = _parse_instructions(d.pop("instructions", UNSET))


        def _parse_max_output_tokens(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        max_output_tokens = _parse_max_output_tokens(d.pop("max_output_tokens", UNSET))


        def _parse_temperature(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        temperature = _parse_temperature(d.pop("temperature", UNSET))


        def _parse_top_p(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        top_p = _parse_top_p(d.pop("top_p", UNSET))


        message_input = cls(
            input_=input_,
            tools=tools,
            workspace_ext_id=workspace_ext_id,
            previous_response_id=previous_response_id,
            model=model,
            stream=stream,
            background=background,
            store=store,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            temperature=temperature,
            top_p=top_p,
        )

        return message_input




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.model_status import ModelStatus





T = TypeVar("T", bound="ModelsHealthStatus")



@_attrs_define
class ModelsHealthStatus:
    """ 
        Attributes:
            application (str):
            models (list[ModelStatus]):
     """

    application: str
    models: list[ModelStatus]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.model_status import ModelStatus
        application = self.application

        models = []
        for models_item_data in self.models:
            models_item = models_item_data.to_dict()
            models.append(models_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "application": application,
            "models": models,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.model_status import ModelStatus
        d = dict(src_dict)
        application = d.pop("application")

        models = []
        _models = d.pop("models")
        for models_item_data in (_models):
            models_item = ModelStatus.from_dict(models_item_data)



            models.append(models_item)


        models_health_status = cls(
            application=application,
            models=models,
        )


        models_health_status.additional_properties = d
        return models_health_status

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.evaluator_llm_config_api_type import EvaluatorLLMConfigApiType
from ..types import UNSET, Unset






T = TypeVar("T", bound="EvaluatorLLMConfig")



@_attrs_define
class EvaluatorLLMConfig:
    """ 
        Attributes:
            api_type (EvaluatorLLMConfigApiType | Unset): The inference type (local or remote). Default:
                EvaluatorLLMConfigApiType.REMOTE.
            enable_thinking (bool | Unset): Enable reasoning/thinking mode. Default: False.
            model_name (str | Unset): The name of the non-reasoning model to be used. Default: 'dummy'.
            system_instruction (str | Unset): The system instruction for chunk evaluation. Default: "You are a chunk
                evaluator. Analyze retrieved chunks to determine which are relevant to the user's query and extract key
                learnings from the RELEVANT chunks only. You must return your response as valid JSON matching the provided
                schema.".
            temperature (float | Unset): Low temperature for consistent evaluation. Default: 0.1.
            max_tokens (int | Unset): Maximum tokens for evaluation response. Default: 20000.
            max_char_size_to_answer (int | Unset): Maximum character size for evaluation context. Default: 100000.
     """

    api_type: EvaluatorLLMConfigApiType | Unset = EvaluatorLLMConfigApiType.REMOTE
    enable_thinking: bool | Unset = False
    model_name: str | Unset = 'dummy'
    system_instruction: str | Unset = "You are a chunk evaluator. Analyze retrieved chunks to determine which are relevant to the user's query and extract key learnings from the RELEVANT chunks only. You must return your response as valid JSON matching the provided schema."
    temperature: float | Unset = 0.1
    max_tokens: int | Unset = 20000
    max_char_size_to_answer: int | Unset = 100000
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        api_type: str | Unset = UNSET
        if not isinstance(self.api_type, Unset):
            api_type = self.api_type.value


        enable_thinking = self.enable_thinking

        model_name = self.model_name

        system_instruction = self.system_instruction

        temperature = self.temperature

        max_tokens = self.max_tokens

        max_char_size_to_answer = self.max_char_size_to_answer


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if api_type is not UNSET:
            field_dict["API_TYPE"] = api_type
        if enable_thinking is not UNSET:
            field_dict["ENABLE_THINKING"] = enable_thinking
        if model_name is not UNSET:
            field_dict["MODEL_NAME"] = model_name
        if system_instruction is not UNSET:
            field_dict["SYSTEM_INSTRUCTION"] = system_instruction
        if temperature is not UNSET:
            field_dict["TEMPERATURE"] = temperature
        if max_tokens is not UNSET:
            field_dict["MAX_TOKENS"] = max_tokens
        if max_char_size_to_answer is not UNSET:
            field_dict["MAX_CHAR_SIZE_TO_ANSWER"] = max_char_size_to_answer

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _api_type = d.pop("API_TYPE", UNSET)
        api_type: EvaluatorLLMConfigApiType | Unset
        if isinstance(_api_type,  Unset):
            api_type = UNSET
        else:
            api_type = EvaluatorLLMConfigApiType(_api_type)




        enable_thinking = d.pop("ENABLE_THINKING", UNSET)

        model_name = d.pop("MODEL_NAME", UNSET)

        system_instruction = d.pop("SYSTEM_INSTRUCTION", UNSET)

        temperature = d.pop("TEMPERATURE", UNSET)

        max_tokens = d.pop("MAX_TOKENS", UNSET)

        max_char_size_to_answer = d.pop("MAX_CHAR_SIZE_TO_ANSWER", UNSET)

        evaluator_llm_config = cls(
            api_type=api_type,
            enable_thinking=enable_thinking,
            model_name=model_name,
            system_instruction=system_instruction,
            temperature=temperature,
            max_tokens=max_tokens,
            max_char_size_to_answer=max_char_size_to_answer,
        )


        evaluator_llm_config.additional_properties = d
        return evaluator_llm_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.notification_type import NotificationType
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime

if TYPE_CHECKING:
  from ..models.user_response import UserResponse





T = TypeVar("T", bound="NotificationResponse")



@_attrs_define
class NotificationResponse:
    """ Notification response model for API and WebSocket.

    Bilateral: both sender and recipient see the same row.
    Client determines perspective: sender == me → I sent it, else I received it.

        Attributes:
            type_ (NotificationType): Notification types - all persisted AND delivered via WebSocket.

                Type is self-descriptive, no need to parse content field.
            external_id (str):
            sender (UserResponse): Standard user representation used across all endpoints.

                Used for: login response, workspace users, contacts (when registered).
                Agents have agt- prefix and may have nullable name/email fields.
            recipient (UserResponse): Standard user representation used across all endpoints.

                Used for: login response, workspace users, contacts (when registered).
                Agents have agt- prefix and may have nullable name/email fields.
            created_at (datetime.datetime):
            updated_at (datetime.datetime):
            workspace_ext_id (None | str | Unset):
            content (None | str | Unset):
            read (bool | Unset):  Default: False.
     """

    type_: NotificationType
    external_id: str
    sender: UserResponse
    recipient: UserResponse
    created_at: datetime.datetime
    updated_at: datetime.datetime
    workspace_ext_id: None | str | Unset = UNSET
    content: None | str | Unset = UNSET
    read: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.user_response import UserResponse
        type_ = self.type_.value

        external_id = self.external_id

        sender = self.sender.to_dict()

        recipient = self.recipient.to_dict()

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        workspace_ext_id: None | str | Unset
        if isinstance(self.workspace_ext_id, Unset):
            workspace_ext_id = UNSET
        else:
            workspace_ext_id = self.workspace_ext_id

        content: None | str | Unset
        if isinstance(self.content, Unset):
            content = UNSET
        else:
            content = self.content

        read = self.read


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "external_id": external_id,
            "sender": sender,
            "recipient": recipient,
            "created_at": created_at,
            "updated_at": updated_at,
        })
        if workspace_ext_id is not UNSET:
            field_dict["workspace_ext_id"] = workspace_ext_id
        if content is not UNSET:
            field_dict["content"] = content
        if read is not UNSET:
            field_dict["read"] = read

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_response import UserResponse
        d = dict(src_dict)
        type_ = NotificationType(d.pop("type"))




        external_id = d.pop("external_id")

        sender = UserResponse.from_dict(d.pop("sender"))




        recipient = UserResponse.from_dict(d.pop("recipient"))




        created_at = isoparse(d.pop("created_at"))




        updated_at = isoparse(d.pop("updated_at"))




        def _parse_workspace_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        workspace_ext_id = _parse_workspace_ext_id(d.pop("workspace_ext_id", UNSET))


        def _parse_content(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        content = _parse_content(d.pop("content", UNSET))


        read = d.pop("read", UNSET)

        notification_response = cls(
            type_=type_,
            external_id=external_id,
            sender=sender,
            recipient=recipient,
            created_at=created_at,
            updated_at=updated_at,
            workspace_ext_id=workspace_ext_id,
            content=content,
            read=read,
        )


        notification_response.additional_properties = d
        return notification_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.filter_plan_llm_config_api_type import FilterPlanLLMConfigApiType
from ..types import UNSET, Unset






T = TypeVar("T", bound="FilterPlanLLMConfig")



@_attrs_define
class FilterPlanLLMConfig:
    r""" Configuration for FilterPlanLLM — pre-agentic context selector.

    Runs before the agent loop to select relevant history messages and documents,
    plus produce a concise goal statement that primes the agent.
    Always uses the fast model (LLM_MODEL_NAME). In auto mode, also recommends
    a model tier (fast/wise/premium) for the agent loop.

        Attributes:
            api_type (FilterPlanLLMConfigApiType | Unset): The inference type (local or remote). Default:
                FilterPlanLLMConfigApiType.REMOTE.
            enable_thinking (bool | Unset): Enable reasoning/thinking mode. Default: False.
            model_name (str | Unset): The name of the non-reasoning model to be used. Default: 'dummy'.
            system_instruction (str | Unset): System prompt for the filter/plan LLM. Default: 'You are a context selector.
                Given a user\'s query, conversation history, and a table of available documents, determine the user\'s goal and
                select relevant context.\n\n## Instructions\n\n1. **goal**: 1-2 concise sentences describing the user\'s intent
                AND your filtering rationale. Explain what documents are needed and why. Examples:\n   - \\"The user is greeting
                the assistant. No documents are needed.\\"\n   - \\"The user wants to find force majeure clauses. Only the three
                contracts are relevant, so I will keep only those.\\"\n   - \\"The user wants a full timeline of the case. Most
                documents are relevant, so I will drop only the unrelated procedural orders.\\"\n\n2. **doc_filter_mode** and
                **doc_ids**: Select which documents the agent needs.\n   - `\\"keep\\"` mode: `doc_ids` lists the ONLY docs to
                keep. Everything else is dropped.\n   - `\\"drop\\"` mode: `doc_ids` lists the ONLY docs to drop. Everything
                else is kept.\n   - Pick whichever mode produces the **shorter `doc_ids` list**.\n   - Doc IDs are in the **Id**
                column of the Available Documents table (e.g. `\\"doc-abc12345\\"`). Copy them **exactly** from the table — do
                not invent, guess, or modify IDs.\n\n   | Scenario | doc_filter_mode | doc_ids |\n
                |----------|----------------|---------|\n   | Need ALL docs | `\\"drop\\"` | `[]` |\n   | Need NO docs |
                `\\"keep\\"` | `[]` |\n   | Need 3 of 100 docs | `\\"keep\\"` | `[\\"id1\\",\\"id2\\",\\"id3\\"]` |\n   | Drop 3
                of 100 docs | `\\"drop\\"` | `[\\"id1\\",\\"id2\\",\\"id3\\"]` |\n\n3. **chat_range_start** and
                **chat_range_end**: 0-based inclusive range of earlier history messages that provide relevant context. The last
                3 messages are always included automatically. Set both to -1 if no earlier history is needed.\n\n## Rules\n\n-
                **Never repeat a doc ID.** Each ID must appear at most once.\n- Before writing `doc_ids`, count how many you
                would keep vs drop, then pick the shorter list.\n- Greetings or meta-questions about the assistant (e.g.
                \\"hello\\", \\"who are you?\\", \\"what can you do?\\"): the agent does NOT need any documents → `\\"keep\\"`
                with `[]`.\n- Ambiguous references like \\"the document\\" or \\"this contract\\" without specifying which: keep
                all → `\\"drop\\"` with `[]`.\n- For follow-up questions, include history messages that establish the topic.\n-
                Always return valid JSON matching the schema.'.
            temperature (float | Unset):  Default: 0.0.
            max_tokens (int | Unset):  Default: 3000.
     """

    api_type: FilterPlanLLMConfigApiType | Unset = FilterPlanLLMConfigApiType.REMOTE
    enable_thinking: bool | Unset = False
    model_name: str | Unset = 'dummy'
    system_instruction: str | Unset = 'You are a context selector. Given a user\'s query, conversation history, and a table of available documents, determine the user\'s goal and select relevant context.\n\n## Instructions\n\n1. **goal**: 1-2 concise sentences describing the user\'s intent AND your filtering rationale. Explain what documents are needed and why. Examples:\n   - \\"The user is greeting the assistant. No documents are needed.\\"\n   - \\"The user wants to find force majeure clauses. Only the three contracts are relevant, so I will keep only those.\\"\n   - \\"The user wants a full timeline of the case. Most documents are relevant, so I will drop only the unrelated procedural orders.\\"\n\n2. **doc_filter_mode** and **doc_ids**: Select which documents the agent needs.\n   - `\\"keep\\"` mode: `doc_ids` lists the ONLY docs to keep. Everything else is dropped.\n   - `\\"drop\\"` mode: `doc_ids` lists the ONLY docs to drop. Everything else is kept.\n   - Pick whichever mode produces the **shorter `doc_ids` list**.\n   - Doc IDs are in the **Id** column of the Available Documents table (e.g. `\\"doc-abc12345\\"`). Copy them **exactly** from the table — do not invent, guess, or modify IDs.\n\n   | Scenario | doc_filter_mode | doc_ids |\n   |----------|----------------|---------|\n   | Need ALL docs | `\\"drop\\"` | `[]` |\n   | Need NO docs | `\\"keep\\"` | `[]` |\n   | Need 3 of 100 docs | `\\"keep\\"` | `[\\"id1\\",\\"id2\\",\\"id3\\"]` |\n   | Drop 3 of 100 docs | `\\"drop\\"` | `[\\"id1\\",\\"id2\\",\\"id3\\"]` |\n\n3. **chat_range_start** and **chat_range_end**: 0-based inclusive range of earlier history messages that provide relevant context. The last 3 messages are always included automatically. Set both to -1 if no earlier history is needed.\n\n## Rules\n\n- **Never repeat a doc ID.** Each ID must appear at most once.\n- Before writing `doc_ids`, count how many you would keep vs drop, then pick the shorter list.\n- Greetings or meta-questions about the assistant (e.g. \\"hello\\", \\"who are you?\\", \\"what can you do?\\"): the agent does NOT need any documents → `\\"keep\\"` with `[]`.\n- Ambiguous references like \\"the document\\" or \\"this contract\\" without specifying which: keep all → `\\"drop\\"` with `[]`.\n- For follow-up questions, include history messages that establish the topic.\n- Always return valid JSON matching the schema.'
    temperature: float | Unset = 0.0
    max_tokens: int | Unset = 3000
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        api_type: str | Unset = UNSET
        if not isinstance(self.api_type, Unset):
            api_type = self.api_type.value


        enable_thinking = self.enable_thinking

        model_name = self.model_name

        system_instruction = self.system_instruction

        temperature = self.temperature

        max_tokens = self.max_tokens


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if api_type is not UNSET:
            field_dict["API_TYPE"] = api_type
        if enable_thinking is not UNSET:
            field_dict["ENABLE_THINKING"] = enable_thinking
        if model_name is not UNSET:
            field_dict["MODEL_NAME"] = model_name
        if system_instruction is not UNSET:
            field_dict["SYSTEM_INSTRUCTION"] = system_instruction
        if temperature is not UNSET:
            field_dict["TEMPERATURE"] = temperature
        if max_tokens is not UNSET:
            field_dict["MAX_TOKENS"] = max_tokens

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _api_type = d.pop("API_TYPE", UNSET)
        api_type: FilterPlanLLMConfigApiType | Unset
        if isinstance(_api_type,  Unset):
            api_type = UNSET
        else:
            api_type = FilterPlanLLMConfigApiType(_api_type)




        enable_thinking = d.pop("ENABLE_THINKING", UNSET)

        model_name = d.pop("MODEL_NAME", UNSET)

        system_instruction = d.pop("SYSTEM_INSTRUCTION", UNSET)

        temperature = d.pop("TEMPERATURE", UNSET)

        max_tokens = d.pop("MAX_TOKENS", UNSET)

        filter_plan_llm_config = cls(
            api_type=api_type,
            enable_thinking=enable_thinking,
            model_name=model_name,
            system_instruction=system_instruction,
            temperature=temperature,
            max_tokens=max_tokens,
        )


        filter_plan_llm_config.additional_properties = d
        return filter_plan_llm_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.retrieval_chunk_tool_args_search_mode_type_0 import RetrievalChunkToolArgsSearchModeType0
from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="RetrievalChunkToolArgs")



@_attrs_define
class RetrievalChunkToolArgs:
    """ Typed arguments for retrieval chunk tool.

        Attributes:
            doc_ext_ids (list[str] | Unset):
            search_mode (None | RetrievalChunkToolArgsSearchModeType0 | Unset):
     """

    doc_ext_ids: list[str] | Unset = UNSET
    search_mode: None | RetrievalChunkToolArgsSearchModeType0 | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_ext_ids: list[str] | Unset = UNSET
        if not isinstance(self.doc_ext_ids, Unset):
            doc_ext_ids = self.doc_ext_ids



        search_mode: None | str | Unset
        if isinstance(self.search_mode, Unset):
            search_mode = UNSET
        elif isinstance(self.search_mode, RetrievalChunkToolArgsSearchModeType0):
            search_mode = self.search_mode.value
        else:
            search_mode = self.search_mode


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if doc_ext_ids is not UNSET:
            field_dict["doc_ext_ids"] = doc_ext_ids
        if search_mode is not UNSET:
            field_dict["search_mode"] = search_mode

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        doc_ext_ids = cast(list[str], d.pop("doc_ext_ids", UNSET))


        def _parse_search_mode(data: object) -> None | RetrievalChunkToolArgsSearchModeType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                search_mode_type_0 = RetrievalChunkToolArgsSearchModeType0(data)



                return search_mode_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RetrievalChunkToolArgsSearchModeType0 | Unset, data)

        search_mode = _parse_search_mode(d.pop("search_mode", UNSET))


        retrieval_chunk_tool_args = cls(
            doc_ext_ids=doc_ext_ids,
            search_mode=search_mode,
        )


        retrieval_chunk_tool_args.additional_properties = d
        return retrieval_chunk_tool_args

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field
import json
from .. import types

from ..types import UNSET, Unset

from ..types import File, FileTypes
from io import BytesIO






T = TypeVar("T", bound="UploadFileBody")



@_attrs_define
class UploadFileBody:
    """ 
        Attributes:
            file (File): The file to upload
            purpose (str): The intended purpose of the uploaded file. Default: 'assistants'.
     """

    file: File
    purpose: str = 'assistants'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        file = self.file.to_tuple()


        purpose = self.purpose


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "file": file,
            "purpose": purpose,
        })

        return field_dict


    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("file", self.file.to_tuple()))



        files.append(("purpose", (None, str(self.purpose).encode(), "text/plain")))




        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))



        return files


    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        file = File(
             payload = BytesIO(d.pop("file"))
        )




        purpose = d.pop("purpose")

        upload_file_body = cls(
            file=file,
            purpose=purpose,
        )


        upload_file_body.additional_properties = d
        return upload_file_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.workspace_role import WorkspaceRole
from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="AddWorkspaceUsersRequest")



@_attrs_define
class AddWorkspaceUsersRequest:
    """ Request to add users to a workspace (POST /workspace/users).

    All invited users receive the same role.

    workspace_key + workspace_ext_id (optional): If provided, the caller supplies the
    SealedBox-encrypted workspace key directly (encrypted with their session public key)
    and the target workspace external ID. This allows sharing any workspace the caller
    holds a key for without needing to open/activate it first. If omitted, the server
    falls back to the current session's active workspace.

    session_pubkey_b64 (optional): If set, the grant is temporary and scoped to the
    recipient's current session (identified by this X25519 public key). The workspace
    key is stored in that session's Redis entry and the workspaceusers row expires when
    the session expires. Omit for a permanent grant (default behaviour).

        Attributes:
            emails (list[str]):
            role (WorkspaceRole | Unset): Role of a user within a workspace.
            workspace_key (None | str | Unset):
            workspace_ext_id (None | str | Unset):
            session_pubkey_b64 (None | str | Unset):
     """

    emails: list[str]
    role: WorkspaceRole | Unset = UNSET
    workspace_key: None | str | Unset = UNSET
    workspace_ext_id: None | str | Unset = UNSET
    session_pubkey_b64: None | str | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        emails = self.emails



        role: str | Unset = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value


        workspace_key: None | str | Unset
        if isinstance(self.workspace_key, Unset):
            workspace_key = UNSET
        else:
            workspace_key = self.workspace_key

        workspace_ext_id: None | str | Unset
        if isinstance(self.workspace_ext_id, Unset):
            workspace_ext_id = UNSET
        else:
            workspace_ext_id = self.workspace_ext_id

        session_pubkey_b64: None | str | Unset
        if isinstance(self.session_pubkey_b64, Unset):
            session_pubkey_b64 = UNSET
        else:
            session_pubkey_b64 = self.session_pubkey_b64


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "emails": emails,
        })
        if role is not UNSET:
            field_dict["role"] = role
        if workspace_key is not UNSET:
            field_dict["workspace_key"] = workspace_key
        if workspace_ext_id is not UNSET:
            field_dict["workspace_ext_id"] = workspace_ext_id
        if session_pubkey_b64 is not UNSET:
            field_dict["session_pubkey_b64"] = session_pubkey_b64

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        emails = cast(list[str], d.pop("emails"))


        _role = d.pop("role", UNSET)
        role: WorkspaceRole | Unset
        if isinstance(_role,  Unset):
            role = UNSET
        else:
            role = WorkspaceRole(_role)




        def _parse_workspace_key(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        workspace_key = _parse_workspace_key(d.pop("workspace_key", UNSET))


        def _parse_workspace_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        workspace_ext_id = _parse_workspace_ext_id(d.pop("workspace_ext_id", UNSET))


        def _parse_session_pubkey_b64(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        session_pubkey_b64 = _parse_session_pubkey_b64(d.pop("session_pubkey_b64", UNSET))


        add_workspace_users_request = cls(
            emails=emails,
            role=role,
            workspace_key=workspace_key,
            workspace_ext_id=workspace_ext_id,
            session_pubkey_b64=session_pubkey_b64,
        )

        return add_workspace_users_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="CreateSubscriptionRequest")



@_attrs_define
class CreateSubscriptionRequest:
    """ Request to create a new subscription checkout session.

        Attributes:
            price_id (str):
     """

    price_id: str





    def to_dict(self) -> dict[str, Any]:
        price_id = self.price_id


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "price_id": price_id,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        price_id = d.pop("price_id")

        create_subscription_request = cls(
            price_id=price_id,
        )

        return create_subscription_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.output_message_status import OutputMessageStatus
from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast

if TYPE_CHECKING:
  from ..models.output_text_content import OutputTextContent





T = TypeVar("T", bound="OutputMessage")



@_attrs_define
class OutputMessage:
    """ A single output message entry in a response.

        Attributes:
            id (str): Message external ID
            status (OutputMessageStatus): Message status
            type_ (Literal['message'] | Unset):  Default: 'message'.
            role (Literal['assistant'] | Unset):  Default: 'assistant'.
            content (list[OutputTextContent] | Unset): Message content blocks
     """

    id: str
    status: OutputMessageStatus
    type_: Literal['message'] | Unset = 'message'
    role: Literal['assistant'] | Unset = 'assistant'
    content: list[OutputTextContent] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.output_text_content import OutputTextContent
        id = self.id

        status = self.status.value

        type_ = self.type_

        role = self.role

        content: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.content, Unset):
            content = []
            for content_item_data in self.content:
                content_item = content_item_data.to_dict()
                content.append(content_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "status": status,
        })
        if type_ is not UNSET:
            field_dict["type"] = type_
        if role is not UNSET:
            field_dict["role"] = role
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.output_text_content import OutputTextContent
        d = dict(src_dict)
        id = d.pop("id")

        status = OutputMessageStatus(d.pop("status"))




        type_ = cast(Literal['message'] | Unset , d.pop("type", UNSET))
        if type_ != 'message'and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'message', got '{type_}'")

        role = cast(Literal['assistant'] | Unset , d.pop("role", UNSET))
        if role != 'assistant'and not isinstance(role, Unset):
            raise ValueError(f"role must match const 'assistant', got '{role}'")

        _content = d.pop("content", UNSET)
        content: list[OutputTextContent] | Unset = UNSET
        if _content is not UNSET:
            content = []
            for content_item_data in _content:
                content_item = OutputTextContent.from_dict(content_item_data)



                content.append(content_item)


        output_message = cls(
            id=id,
            status=status,
            type_=type_,
            role=role,
            content=content,
        )


        output_message.additional_properties = d
        return output_message

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
